@@ -16,8 +16,10 @@ from ruamel.yaml.error import (MarkedYAMLError, MarkedYAMLFutureWarning,
 from ruamel.yaml.nodes import *                               # NOQA
 from ruamel.yaml.nodes import (SequenceNode, MappingNode, ScalarNode)
 from ruamel.yaml.compat import (utf8, builtins_module, to_str, PY2, PY3,  # NOQA
-                                ordereddict, text_type, nprint, nprintf, version_tnf,
-                                Hashable, MutableSequence, MutableMapping)
+                                text_type, nprint, nprintf, version_tnf)
+from ruamel.yaml.compat import ordereddict, Hashable, MutableSequence  # type: ignore
+from ruamel.yaml.compat import MutableMapping  # type: ignore
+
 from ruamel.yaml.comments import *                               # NOQA
 from ruamel.yaml.comments import (CommentedMap, CommentedOrderedMap, CommentedSet,
                                   CommentedKeySeq, CommentedSeq, TaggedScalar,
@@ -141,19 +143,31 @@ class BaseConstructor(object):
             #     None, None, 'found unconstructable recursive node', node.start_mark
             # )
         self.recursive_objects[node] = None
+        data = self.construct_non_recursive_object(node)
+
+        self.constructed_objects[node] = data
+        del self.recursive_objects[node]
+        if deep:
+            self.deep_construct = old_deep
+        return data
+
+    def construct_non_recursive_object(self, node, tag=None):
+        # type: (Any, Optional[str]) -> Any
         constructor = None  # type: Any
         tag_suffix = None
-        if node.tag in self.yaml_constructors:
-            constructor = self.yaml_constructors[node.tag]
+        if tag is None:
+            tag = node.tag
+        if tag in self.yaml_constructors:
+            constructor = self.yaml_constructors[tag]
         else:
             for tag_prefix in self.yaml_multi_constructors:
-                if node.tag.startswith(tag_prefix):
-                    tag_suffix = node.tag[len(tag_prefix) :]
+                if tag.startswith(tag_prefix):
+                    tag_suffix = tag[len(tag_prefix) :]
                     constructor = self.yaml_multi_constructors[tag_prefix]
                     break
             else:
                 if None in self.yaml_multi_constructors:
-                    tag_suffix = node.tag
+                    tag_suffix = tag
                     constructor = self.yaml_multi_constructors[None]
                 elif None in self.yaml_constructors:
                     constructor = self.yaml_constructors[None]
@@ -175,10 +189,6 @@ class BaseConstructor(object):
                     pass
             else:
                 self.state_generators.append(generator)
-        self.constructed_objects[node] = data
-        del self.recursive_objects[node]
-        if deep:
-            self.deep_construct = old_deep
         return data
 
     def construct_scalar(self, node):
@@ -362,6 +372,10 @@ class SafeConstructor(BaseConstructor):
             key_node, value_node = node.value[index]
             if key_node.tag == u'tag:yaml.org,2002:merge':
                 if merge:  # double << key
+                    if self.allow_duplicate_keys:
+                        del node.value[index]
+                        index += 1
+                        continue
                     args = [
                         'while constructing a mapping',
                         node.start_mark,
@@ -434,6 +448,8 @@ class SafeConstructor(BaseConstructor):
     bool_values = {
         u'yes': True,
         u'no': False,
+        u'y': True,
+        u'n': False,
         u'true': True,
         u'false': False,
         u'on': True,
@@ -943,7 +959,7 @@ class Constructor(SafeConstructor):
             elif state:
                 slotstate.update(state)
             for key, value in slotstate.items():
-                setattr(object, key, value)
+                setattr(instance, key, value)
 
     def construct_python_object(self, suffix, node):
         # type: (Any, Any) -> Any
@@ -1295,9 +1311,10 @@ class RoundTripConstructor(SafeConstructor):
             if not templated_id(node.anchor):
                 seqtyp.yaml_set_anchor(node.anchor)
         for idx, child in enumerate(node.value):
-            ret_val.append(self.construct_object(child, deep=deep))
             if child.comment:
                 seqtyp._yaml_add_comment(child.comment, key=idx)
+                child.comment = None  # if moved to sequence remove from child
+            ret_val.append(self.construct_object(child, deep=deep))
             seqtyp._yaml_set_idx_line_col(
                 idx, [child.start_mark.line, child.start_mark.column]
             )
@@ -1330,6 +1347,10 @@ class RoundTripConstructor(SafeConstructor):
             key_node, value_node = node.value[index]
             if key_node.tag == u'tag:yaml.org,2002:merge':
                 if merge_map_list:  # double << key
+                    if self.allow_duplicate_keys:
+                        del node.value[index]
+                        index += 1
+                        continue
                     args = [
                         'while constructing a mapping',
                         node.start_mark,
@@ -1447,7 +1468,6 @@ class RoundTripConstructor(SafeConstructor):
                     )
             value = self.construct_object(value_node, deep=deep)
             if self.check_mapping_key(node, key_node, maptyp, key, value):
-
                 if key_node.comment and len(key_node.comment) > 4 and key_node.comment[4]:
                     if last_value is None:
                         key_node.comment[0] = key_node.comment.pop(4)
@@ -1708,6 +1728,7 @@ class RoundTripConstructor(SafeConstructor):
             delta = datetime.timedelta(hours=tz_hour, minutes=tz_minute)
             if values['tz_sign'] == '-':
                 delta = -delta
+        # shold check for NOne and solve issue 366 should be tzinfo=delta)
         if delta:
             dt = datetime.datetime(year, month, day, hour, minute)
             dt -= delta
